# pylint: disable=missing-docstring

import unittest


class TestCli(unittest.TestCase):

    def test_dummy(self):
        """
        missing, yep, you are right. This is just big black hole. This works or not working.
        Sorry about buggy plugin. Challence: if you care about this plugin, please write ONE test,
        I'll write rest of them!
        """
